import { fs, semver, lodash } from "@mango-scripts/utils";
import { run } from "../../utils/index.mjs";
const { uniq } = lodash;
const getLatestTagForPkg = async (pkg) => {
  const tags = (await run("git", ["tag"], { stdio: "pipe" })).stdout.split("\n").filter((tag) => tag.startsWith(`${pkg.pkgName}@`)).map((tag) => tag.replace(`${pkg.pkgName}@`, "")).filter(semver.valid).sort(semver.rcompare);
  return tags[0] ? `${pkg.pkgName}@${tags[0]}` : void 0;
};
const logRecentCommitListForPkg = async (pkg) => {
  const latestTag = await getLatestTagForPkg(pkg);
  const logRange = latestTag ? `${latestTag}..HEAD` : "HEAD";
  const commitFormat = [
    "--no-pager",
    "log",
    logRange,
    "--color",
    "--graph",
    "--pretty=format:%C(Magenta)[%cd]%Cgreen(%cr)%C(bold blue)<%an>%Creset %Cred%h%Creset -%C(yellow)%d%Creset %s",
    "--date=format:%Y-%m-%d %H:%M:%S",
    "--abbrev-commit",
    "--",
    `packages/${pkg.pkgDir}`
  ];
  return (await run("git", commitFormat, { stdio: "pipe" }))?.stdout?.split("\n")?.filter(Boolean) || [];
};
const getDepNameList = async ({ pkgJsonFilePath }) => {
  if (!await fs.pathExists(pkgJsonFilePath)) return [];
  const { dependencies = {}, devDependencies = {} } = await fs.readJson(pkgJsonFilePath);
  return uniq([...Object.keys(dependencies), ...Object.keys(devDependencies)]);
};
const generateDependencyMap = async (allPkgList) => {
  const pkgLevelCache = /* @__PURE__ */ new Map();
  const getPkgLevels = async (pkg, level = 1) => {
    if (pkgLevelCache.has(pkg.pkgName)) {
      return pkgLevelCache.get(pkg.pkgName) || [];
    }
    const depNameList = await getDepNameList(pkg);
    const depsWithLevel = [level];
    for (const depName of depNameList) {
      const depPkg = allPkgList.find((p) => p.pkgName === depName);
      if (depPkg) {
        const depLevels = await getPkgLevels(depPkg, level + 1);
        depsWithLevel.push(...depLevels);
      }
    }
    pkgLevelCache.set(pkg.pkgName, uniq(depsWithLevel));
    return pkgLevelCache.get(pkg.pkgName) || [];
  };
  const dependencyMap = [];
  for (const pkg of allPkgList) {
    const deps = [];
    const depNameList = await getDepNameList(pkg);
    for (const depName of depNameList) {
      const depPkg = allPkgList.find((p) => p.pkgName === depName);
      if (depPkg) {
        const depLevels = await getPkgLevels(depPkg);
        deps.push({ pkg: depPkg, level: depLevels });
      }
    }
    dependencyMap.push({ pkg, deps });
  }
  return dependencyMap;
};
const addChangedDepInfoForPkgList = async (pkgListWithCommitList, pkgList) => {
  const dependency = await generateDependencyMap(pkgList);
  return pkgList.map((pkg) => {
    const pkgDependency = dependency.find(
      ({ pkg: { pkgName } }) => pkgName === pkg.pkgName
    );
    return {
      ...pkg,
      changedDep: pkgDependency && pkgDependency?.deps.filter(
        (dep) => pkgListWithCommitList.findIndex(
          (d) => dep.pkg.pkgName === d.pkgName
        )
      ),
      commitList: pkgListWithCommitList.find((dep) => dep.pkgName === pkg.pkgName)?.commitList ?? []
    };
  });
};
const getDiffPkgList = async (pkgList) => {
  const pkgListWithCommitList = (await Promise.all(
    pkgList.map(async (pkg) => {
      return { ...pkg, commitList: await logRecentCommitListForPkg(pkg) };
    })
  )).filter((pkg) => {
    return pkg.commitList.length > 0;
  });
  return await addChangedDepInfoForPkgList(pkgListWithCommitList, pkgList);
};
export {
  getDiffPkgList,
  logRecentCommitListForPkg
};
